package org.gcube.dataharvest.dao;

import java.util.Date;
import java.util.List;

import org.gcube.dataharvest.datamodel.Harvest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DatabaseManager {
	private static Logger logger = LoggerFactory.getLogger(DatabaseManager.class);
	private boolean testMode = false;

	public DatabaseManager(boolean testMode) {
		this.testMode = testMode;
	}

	public DatabaseManager() {
		this.testMode = false;
	}

	public void setTestMode(boolean testMode) {
		this.testMode = testMode;
	}

	public boolean isTestMode() {
		return testMode;
	}

	public void insertMonthlyData(Date from, Date to, List<Harvest> data, boolean updateFlag) {
		Dao dao = null;
		try {
			dao = dbConnect(testMode);
			dao.insertMonthlyMeasure(data, from, to, updateFlag);
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage());
		} finally {
			if (dao != null) {
				try {
					dao.disconnect();
				} catch (DaoException e) {
					logger.error(e.getLocalizedMessage());
				}
			}
		}
	}

	public String[] getActiveVres() {
		Dao dao = null;
		try {
			dao = dbConnect(testMode);
			return dao.getActiveVres();
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage());
			return null;
		} finally {
			if (dao != null) {
				try {
					dao.disconnect();
				} catch (DaoException e) {
					logger.error(e.getLocalizedMessage());
				}
			}
		}
	}

	public Dao dbConnect(boolean testMode) throws DaoException {
		DatabaseDataExplorer dde = new DatabaseDataExplorer();
		dde.setTestMode(testMode);
		DatabaseConnectionData dcd = dde.retrieveDatabaseInfo();
		Dao dao = new Dao();
		dao.init();
		dao.connect(dcd.getUrl(), dcd.getUser(), dcd.getPassword());
		return dao;
	}

}
