package org.gcube.dataharvest.harvester;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;

import org.gcube.accounting.datamodel.aggregation.AggregatedServiceUsageRecord;
import org.gcube.accounting.analytics.Filter;
import org.gcube.accounting.analytics.Info;
import org.gcube.accounting.analytics.TemporalConstraint;
import org.gcube.accounting.analytics.TemporalConstraint.AggregationMode;
import org.gcube.accounting.analytics.persistence.AccountingPersistenceQuery;
import org.gcube.accounting.analytics.persistence.AccountingPersistenceQueryFactory;
import org.gcube.accounting.datamodel.usagerecords.ServiceUsageRecord;

import java.util.SortedMap;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.Iterator;

import org.gcube.dataharvest.datamodel.Harvest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MethodInvocationHarvester extends BasicHarvester {

	private static Logger logger = LoggerFactory.getLogger(MethodInvocationHarvester.class);

	public MethodInvocationHarvester(String start, String end) throws ParseException {
		super(start, end);
	}

	public MethodInvocationHarvester(Date start, Date end) throws ParseException {
		super(start, end);
	}

	@Override
	public List<Harvest> getData() throws Exception {
		try {
			logger.debug("MethodInvocationHarvester::getData()");
			ArrayList<Harvest> data = new ArrayList<Harvest>();
			AccountingPersistenceQuery accountingPersistenceQuery = AccountingPersistenceQueryFactory.getInstance();
			Calendar start = Calendar.getInstance();
			Calendar end = Calendar.getInstance();
			start.setTime(startDate);
			end.setTime(endDate);
			TemporalConstraint temporalConstraint = new TemporalConstraint(start.getTimeInMillis(),
					end.getTimeInMillis(), AggregationMode.MONTHLY);

			List<Filter> filters = new ArrayList<>();
			filters.add(new Filter(ServiceUsageRecord.CALLED_METHOD, "tag"));

			List<String> contexts = new ArrayList<>();
			contexts.add("/d4science.research-infrastructures.eu/SoBigData/TagMe");

			logger.debug("MethodInvocationHarvester::getData()::getContextTimeSeries");
			SortedMap<Filter, SortedMap<Calendar, Info>> result = accountingPersistenceQuery.getContextTimeSeries(
					AggregatedServiceUsageRecord.class, temporalConstraint, filters, contexts, true);
			if (result == null) {
				logger.error("No data found.");
			} else {
				Set<Filter> ks = result.keySet();
				if (ks != null) {
					Iterator<Filter> ksi = ks.iterator();
					while (ksi.hasNext()) {
						// System.out.println("" + ksi.next().toString());
						logger.debug("Filter: " + ksi.next().toString());
					}
				}
			}

			return data;
		} catch (Exception x) {
			StackTraceElement[] ste = x.getStackTrace();
			String errorMessage = "MethodInvocationHarvester: " + x.getLocalizedMessage();
			for (StackTraceElement s : ste) {
				errorMessage += "\n" + s.toString();
			}
			logger.error(errorMessage);
			throw x;
		}
	}/**/

	/* used only for test */
	public static void main(String[] args) {
		String ROOT = "18fed2d9-030b-4c77-93af-af2015d945f7-843339462";
		String GCUBE = "32542bb3-b04b-4112-9af1-7fac049d33cd-98187548";
		String TAGME = "308cf6bd-3cbb-4267-a300-0cf5ae122bc9-843339462";
		String TAGME2 = "9684c4a6-e542-44e1-a39c-4bdcf04befa8-843339462";
		String SOBIGDATA_EU = "6af6ee16-e357-4d2d-ba14-2c5984ab4e02-843339462";
		try {
			MethodInvocationHarvester m = new MethodInvocationHarvester("2018-03-01 00:00:00", "2018-03-31 23:59:59");
			MethodInvocationHarvester.setContext(SOBIGDATA_EU);
			System.out.println(MethodInvocationHarvester.getCurrentContext());
			Properties props = m.readServiceEndpoint();
			System.out.println("Found " + props.size() + " properties.");
			Set keys = props.keySet();
			for(Object key : keys) {
				System.out.println((String)key + " :: " + props.getProperty((String) key));
			}
	
			m.getData();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			System.out.println(e.getLocalizedMessage());
			e.printStackTrace();
		}
	}
	/**/

}
