package org.gcube.dataharvest.harvester;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Iterator;
import java.util.Locale;
import java.util.Properties;

import org.apache.http.client.utils.URLEncodedUtils;
import org.gcube.dataharvest.datamodel.Harvest;
import org.gcube.dataharvest.utils.Utils;
import org.gcube.portlets.user.urlshortener.UrlEncoderUtil;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.quartz.utils.StringKeyDirtyFlagMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ResourceCatalogueHarvester extends BasicHarvester {
	private static Logger logger = LoggerFactory.getLogger(ResourceCatalogueHarvester.class);
	private int cityOfCitizensCounter = 0, migrationStudiesCounter = 0, societalDebatesCounter = 0,
			wellBeingAndEconomyCounter = 0;

	public ResourceCatalogueHarvester(String start, String end) throws ParseException {
		super(start, end);
	}

	public ResourceCatalogueHarvester(Date start, Date end) throws ParseException {
		super(start, end);
	}

	@Override
	public List<Harvest> getData() throws Exception {
		ArrayList<Harvest> data = new ArrayList<Harvest>();
		List<Harvest> dataDeliverable = getDataDeliverable();
		for (Harvest harvest : dataDeliverable) {
			data.add(harvest);
		}

		List<Harvest> dataMethod = getDataMethod();
		for (Harvest harvest : dataMethod) {
			data.add(harvest);
		}

		List<Harvest> dataDataset = getDataDataset();
		for (Harvest harvest : dataDataset) {
			data.add(harvest);
		}

		List<Harvest> dataApplication = getDataApplication();
		for (Harvest harvest : dataApplication) {
			data.add(harvest);
		}

		return data;
	}

	public List<Harvest> getDataDeliverable() throws Exception {
		String json = executeQuery("Deliverable");
		return buildList(json, Harvest.NEW_CATALOGUE_DELIVERABLES);
	}

	public List<Harvest> getDataMethod() throws Exception {
		String json = executeQuery("Method");
		return buildList(json, Harvest.NEW_CATALOGUE_METHODS);
	}

	public List<Harvest> getDataDataset() throws Exception {
		String json = executeQuery("Dataset");
		return buildList(json, Harvest.NEW_CATALOGUE_DATASETS);
	}

	public List<Harvest> getDataApplication() throws Exception {
		String json = executeQuery("Application");
		return buildList(json, Harvest.NEW_CATALOGUE_APPLICATIONS);
	}

	private List<Harvest> buildList(String json, int dataType) throws Exception {
		ArrayList<Harvest> data = new ArrayList<Harvest>();
		JSONObject jsonObject = new JSONObject(json);

		JSONObject responseHeader = jsonObject.getJSONObject("responseHeader");
		int status = responseHeader.getInt("status");
		if (status != 0) {
			String err = "Query Deliverable in error: status " + status;
			logger.error(err);
			throw new Exception(err, null);
		}

		JSONObject response = jsonObject.getJSONObject("response");

		int numFound = response.getInt("numFound");
		Harvest h = new Harvest(dataType, "/d4science.research-infrastructures.eu/SoBigData/ResourceCatalogue",
				numFound);
		logger.debug(h.toString());
		data.add(h);
		if (numFound > 0) {

			JSONArray docs = response.getJSONArray("docs");
			for (Object item : docs) {
				JSONObject doc = (JSONObject) item;
				try {
					JSONArray groups = doc.getJSONArray("groups");
					Iterator<Object> git = groups.iterator();
					while (git.hasNext()) {
						String groupItem = (String) git.next();
						counterByGroup(groupItem);
					}
				} catch (JSONException x) {
					logger.debug("Document without groups");
				}
			}

			h = new Harvest(dataType, "/d4science.research-infrastructures.eu/SoBigData/CityOfCitizens",
					cityOfCitizensCounter);
			logger.debug(h.toString());
			data.add(h);
			h = new Harvest(dataType, "/Migration Studies", migrationStudiesCounter);
			logger.debug(h.toString());
			data.add(h);
			h = new Harvest(dataType, "/d4science.research-infrastructures.eu/SoBigData/SocietalDebates",
					societalDebatesCounter);
			logger.debug(h.toString());
			data.add(h);
			h = new Harvest(dataType, "/d4science.research-infrastructures.eu/SoBigData/WellBeingAndEconomy",
					wellBeingAndEconomyCounter);
			logger.debug(h.toString());
			data.add(h);

		}

		return data;

	}

	private String executeQuery(String fqSubString) throws Exception {
		String query = "https://ckan-solr-d4s.d4science.org/solr/sobigdata/select?";
		String q = UrlEncoderUtil.encodeQuery("metadata_created:[" + Utils.dateToStringWithTZ(startDate) + " TO "
				+ Utils.dateToStringWithTZ(endDate) + "]");
		query += "q=" + q;
		String fq = UrlEncoderUtil.encodeQuery("extras_systemtype:\"SoBigData.eu: " + fqSubString + "\"");
		query += "&fq=" + fq + "&wt=json&indent=true";
		logger.debug(query);

		String json = Utils.getJson(query);
		// logger.debug(json);

		return json;
	}

	private void counterByGroup(String groupName) {
		cityOfCitizensCounter = migrationStudiesCounter = societalDebatesCounter = wellBeingAndEconomyCounter = 0;
		switch (groupName) {
		case "city-of-citizens-group":
			cityOfCitizensCounter++;
			break;
		case "migration-studies":
			migrationStudiesCounter++;
			break;
		case "societal-debates-group":
			societalDebatesCounter++;
			break;
		case "well-being-and-economy-group":
			wellBeingAndEconomyCounter++;
			break;
		}
	}

	/*
	 * used only for test public static void main(String[] args) { Calendar from
	 * = Calendar.getInstance(); from.set(Calendar.DAY_OF_MONTH, 1);
	 * from.set(Calendar.MONTH, Calendar.NOVEMBER); from.set(Calendar.YEAR,
	 * 2017); from.set(Calendar.HOUR_OF_DAY, 0); from.set(Calendar.MINUTE, 0);
	 * from.set(Calendar.SECOND, 0);
	 * 
	 * Calendar to = Calendar.getInstance(); to.set(Calendar.DAY_OF_MONTH, 30);
	 * to.set(Calendar.MONTH, Calendar.NOVEMBER); to.set(Calendar.YEAR, 2017);
	 * to.set(Calendar.HOUR_OF_DAY, 23); to.set(Calendar.MINUTE, 59);
	 * to.set(Calendar.SECOND, 59);
	 * 
	 * try { ResourceCatalogueHarvester a = new
	 * ResourceCatalogueHarvester(from.getTime(), to.getTime()); List<Harvest> l
	 * = a.getData(); for(Harvest h : l) { System.out.println(h.toString()); }
	 * // System.out.println(Utils.dateToString(a.startDate)); //
	 * System.out.println(Utils.dateToStringWithTZ(a.endDate)); } catch
	 * (Exception e) { // TODO Auto-generated catch block e.printStackTrace(); }
	 * }
	 */
}
